\name{pathWave.run}
\alias{pathWave.run}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
pathWave.run
}
\description{
The PathWave main procedure for processing expression data.
pathWave.run() evaluates changes in expression levels mapped
to a optimized 2D grid representation of metabolic pathways
to identify pathways with significant patterns of altered reactions.
}
\usage{
pathWave.run(configfile="pathwave.run.conf", preprocessed.tag=NULL,
	     input.exprdata=NULL, input.sampleclasses=NULL,
             param.kegg.only_metabolism=NULL,
	     param.ztransform=NULL, param.numperm=NULL,
	     param.pvalue.correction.method=NULL,
	     param.pvalue.threshold=NULL, param.filter.size=NULL,
	     output.file.prefix=NULL, verbose=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{configfile}{
(Optional) Configuration file containing specifications for the remaining
arguments (including mandatory ones); arguments directly specified with
the function all take precedence over those specified in the file.
Arguments will be read from the configuration file only if NULL has been
passed for them to the function call. Format of the configuration file:\cr
   <argument>=<value>\cr
For example:\cr
   preprocessed.tag="KEGG.hsa"
}
  \item{preprocessed.tag}{
(Mandatory) ID tag of the preprocessed pathway and optimal grid data
(e.g. "KEGG.hsa").
}
  \item{input.exprdata}{
(Mandatory) Expression data. Can be passed as a data.frame containing a
matrix (row names must be Entrez gene IDs, column names are sample IDs).\cr
Alternatively, a file name from which to load the expression data can
be specified. Required file format: Tab-separated vector (TSV); the header
line must contain _only_ sample IDs and data lines must have an additional
preceding field containing the Entrez gene ID.
}
  \item{input.sampleclasses}{
(Mandatory) Definition of two samples classes for the expression data. This
may be one of the following three:\cr
1) a factor matching exactly the columns in the expression data;\cr
2) a data.frame containing a table with two columns (1=sample ID, 2=class);\cr
3) the name of a file containing a mapping from sample ID to class as TSV.\cr
Note: For both 2) and 3), having precise sample ID, also a subset (in
arbitrary order) of the full expression data may be specified.
}
  \item{param.kegg.only_metabolism}{
If set to TRUE and KEGG pathways are used (preprocessed.tag starts
with "KEGG", then only metabolic pathways will be processed. All pathways
regarding other cellular processes (e.g. signaling, DNA repair, etc.) will
be ignored. If preprocessed.tag does not start with
"KEGG", i.e. if we don't use KEGG pathways, this parameter will be ignored.
If the parameter is neither specified in the function call nor the
configuration file, is will be set to TRUE, and therefore
non-metabolic KEGG pathways will be ignored by default!
}
  \item{param.ztransform}{
Logical parameter specifying whether expression data needs to be z-transformed.
If neither specified in the function call nor the configuration file, the
default value (TRUE) will be assumed. Switch this off, if your expression data
has already been z-transformed.
}
  \item{param.numperm}{
Number of permutations to perform for statistical significance testing. If
neither specified in the function call nor the configuration file, the
default value (1000) will be used.
}
  \item{param.pvalue.correction.method}{
Method for multiple testing correction; see ?p.adjust for possible methods.
If neither specified in the function call nor the configuration file, the
default method (Bonferroni) will be used.
}
  \item{param.pvalue.threshold}{
P-value threshold for multiple testing. If neither specified in the function
call nor the configuration file, the default value (0.05) will be used.
}
  \item{param.filter.size}{
Specifies the minimum number of differentially expressed reactions and genes
required for a pathway to be considered as significanly altered. Pathways
that have less differentially expressed reactions or less genes will not
be returned even if they have significant p-values. If neither specified
in the function call nor the configuration file, the default value (3) will be
used.
}
  \item{output.file.prefix}{
(Optional) Prefix for output files. If specified, three output files 
will be written to the working directory:\cr
1) <prefix>-pw-results.rda: the PathWave object "pwres" (see return value below).\cr
2) <prefix>-pw-results_table.tsv: filtered, significant results as a human readable TSV table.
}
  \item{verbose}{
Verbose information on processing; default: TRUE.
}
}
%\details{
%%  ~~ If necessary, more details than the description above ~~
%}
\value{
A list containing the complete and filteredi/significant results (corresponding 
to the PathWave object "pwres" saved with output.file.prefix; see above).\cr
The list contains\cr
..$results.all: results for all pathways (no filtering; uncorrected p-values)\cr
..$results.filtered: only filtered, significant pathways\cr
..$results.table: human readable summary for filtered, significant pathways
}
\references{
Schramm et al. (2010) Bioinformatics 26(9):1225-1231.\cr
Piro et al. (unpublished)\cr
http://www.ichip.de/software/pathwave.html
}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
PathWave
}
%\examples{
%%
%}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{manip}
\keyword{graphs}
\keyword{optimize}
